use std::{
    fs::{create_dir_all, File},
    io::{Read, Write},
    path::PathBuf,
};

use crate::ip_location::IpLocation;
use directories::ProjectDirs;
pub fn save_to_cache(ip: &str, data: &IpLocation) {
    let cache_dir = get_cache_dir();
    if cache_dir.is_none() {
        return;
    }

    let mut cache_dir = cache_dir.unwrap();
    cache_dir.push(format!("{}.{}", ip, "json"));

    let json = serde_json::to_string(data).expect("Error al serializar a JSON");
    let mut file = File::create(&cache_dir).expect("Error al crear el archivo");
    file.write_all(json.as_bytes())
        .expect("Error al escribir en el archivo");
}

pub fn get_from_cache(ip: &str) -> Option<IpLocation> {
    let cache_dir = get_cache_dir();
    if cache_dir.is_none() {
        return None;
    }

    let mut cache_dir = cache_dir.unwrap();
    cache_dir.push(format!("{}.{}", ip, "json"));

    if !is_in_cache(&cache_dir) {
        return None;
    }

    let mut file = File::open(&cache_dir).expect("Error al abrir el archivo");
    let mut content = String::new();
    file.read_to_string(&mut content)
        .expect("Error al leer el archivo");

    let ip_location: IpLocation =
        serde_json::from_str(&content).expect("Error al deserializar JSON");
    Some(ip_location)
}

fn is_in_cache(path: &PathBuf) -> bool {
    path.is_file()
}

fn get_cache_dir() -> Option<PathBuf> {
    let dir = ProjectDirs::from("com", "midefos", "open-meteo-cli");
    if dir.is_some() {
        let project_dir = dir.unwrap();

        let cache_dir = project_dir.cache_dir();
        if !cache_dir.exists() {
            let res = create_dir_all(cache_dir);
            if res.is_err() {
                return None;
            }
        }

        Some(cache_dir.to_owned())
    } else {
        None
    }
}
